from importlib import import_module

from django.core.exceptions import ImproperlyConfigured


def import_by_path(dotted_path, error_prefix=''):
    """
    Import a dotted module path and return the attribute/class designated by
    the last name in the path. Raise ImproperlyConfigured if something goes
    wrong.

    .. warning::
        .. deprecated:: Django 1.6

        Function :py:func:`django.utils.module_loading.import_by_path` has
        been added in Django 1.6.
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError:
        raise ImproperlyConfigured("%s%s doesn't look like a module path" % (
            error_prefix, dotted_path))
    try:
        module = import_module(module_path)
    except ImportError as e:
        msg = '%sError importing module %s: "%s"' % (
            error_prefix, module_path, e)
        raise ImproperlyConfigured(msg)
    try:
        attr = getattr(module, class_name)
    except AttributeError:
        raise ImproperlyConfigured('%sModule "%s" does not define a "%s" \
                                   attribute/class' %
                                   (error_prefix, module_path, class_name))
    return attr



from unittest import TestCase

import embed_video


class EmbedVideoTestCase(TestCase):
    def test_release(self):
        embed_video.VERSION = ('a', 'b', 'c', 'd')
        self.assertEqual('a.b.c', embed_video.get_release())

    def test_version(self):
        embed_video.VERSION = ('a', 'b', 'c', 'd')
        self.assertEqual('a.b', embed_video.get_version())



from django import forms
from django.utils.safestring import mark_safe

from .backends import detect_backend
from .fields import EmbedVideoField


class AdminVideoWidget(forms.TextInput):
    """
    Widget for video input in administration. If empty it works just like
    :py:class:`django.forms.TextInput`. Otherwise it renders embedded video
    together with input field.

    .. todo::

        Django 1.6 provides better parent for this widget -
        :py:class:`django.forms.URLInput`.

    """

    output_format = u'<div style="float:left" class="video">' \
                    u'{video}<br />{input}</div>' \
                    u'<hr style="visibility: hidden; clear:both">'

    def __init__(self, attrs=None):
        default_attrs = {'size': '40'}

        if attrs:
            default_attrs.update(attrs)

        super(AdminVideoWidget, self).__init__(default_attrs)

    def render(self, name, value='', attrs=None, size=(420, 315)):
        output = super(AdminVideoWidget, self).render(name, value, attrs)

        if value:
            try:
                backend = detect_backend(value)
            except:
                pass
            else:
                output = self.output_format.format(
                    video=backend.get_embed_code(*size),
                    input=output,
                )

        return mark_safe(output)


class AdminVideoMixin(object):
    """
    Mixin using :py:class:`AdminVideoWidget` for fields with
    :py:class:`~embed_video.fields.EmbedVideoField`.

    Usage::

        from django.contrib import admin
        from embed_video.admin import AdminVideoMixin
        from .models import MyModel

        class MyModelAdmin(AdminVideoMixin, admin.ModelAdmin):
            pass

        admin.site.register(MyModel, MyModelAdmin)

    """

    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, EmbedVideoField):
            return db_field.formfield(widget=AdminVideoWidget)

        return super(AdminVideoMixin, self) \
                .formfield_for_dbfield(db_field, **kwargs)



from unittest import TestCase

from ..backends import detect_backend, YoutubeBackend, VimeoBackend, \
        SoundCloudBackend, UnknownBackendException, \
        VideoDoesntExistException, UnknownIdException


class BackendTestMixin(object):
    def test_detect(self):
        for url in self.urls:
            backend = detect_backend(url[0])
            self.assertIsInstance(backend, self.instance)

    def test_code(self):
        for url in self.urls:
            backend = self.instance(url[0])
            code = backend.get_code()
            self.assertEqual(code, url[1])


class VideoBackendTestCase(TestCase):
    unknown_backend_urls = (
        'http://myurl.com/?video=http://www.youtube.com/watch?v=jsrRJyHBvzw',
        'http://myurl.com/?video=www.youtube.com/watch?v=jsrRJyHBvzw',
        'http://youtube.com.myurl.com/watch?v=jsrRJyHBvzw',
        'http://vimeo.com.myurl.com/72304002',
    )

    def test_detect_bad_urls(self):
        for url in self.unknown_backend_urls:
            self.assertRaises(UnknownBackendException, detect_backend, url)


class YoutubeBackendTestCase(BackendTestMixin, TestCase):
    urls = (
        ('http://youtu.be/jsrRJyHBvzw', 'jsrRJyHBvzw'),
        ('http://youtu.be/n17B_uFF4cA', 'n17B_uFF4cA'),
        ('http://youtu.be/t-ZRX8984sc', 't-ZRX8984sc'),
        ('https://youtu.be/t-ZRX8984sc', 't-ZRX8984sc'),
        ('http://youtube.com/watch?v=jsrRJyHBvzw', 'jsrRJyHBvzw'),
        ('https://youtube.com/watch?v=jsrRJyHBvzw', 'jsrRJyHBvzw'),
        ('http://www.youtube.com/v/0zM3nApSvMg?rel=0', '0zM3nApSvMg'),
        ('https://www.youtube.com/v/0zM3nApSvMg?rel=0', '0zM3nApSvMg'),
        ('http://www.youtube.com/embed/0zM3nApSvMg?rel=0', '0zM3nApSvMg'),
        ('https://www.youtube.com/embed/0zM3nApSvMg?rel=0', '0zM3nApSvMg'),
        ('http://www.youtube.com/watch?v=jsrRJyHBvzw', 'jsrRJyHBvzw'),
        ('https://www.youtube.com/watch?v=t-ZRX8984sc', 't-ZRX8984sc'),
        ('http://www.youtube.com/watch?v=iwGFalTRHDA&feature=related', 'iwGFalTRHDA'),
        ('https://www.youtube.com/watch?v=iwGFalTRHDA&feature=related', 'iwGFalTRHDA'),
        ('http://www.youtube.com/watch?feature=player_embedded&v=2NpZbaAIXag', '2NpZbaAIXag'),
        ('https://www.youtube.com/watch?feature=player_embedded&v=2NpZbaAIXag', '2NpZbaAIXag'),
    )

    instance = YoutubeBackend

    def test_youtube_keyerror(self):
        """ Test for issue #7 """
        self.assertRaises(UnknownIdException, YoutubeBackend,
                          'http://youtube.com/watch?id=5')


class VimeoBackendTestCase(BackendTestMixin, TestCase):
    urls = (
        ('http://vimeo.com/72304002', '72304002'),
        ('https://vimeo.com/72304002', '72304002'),
        ('http://www.vimeo.com/72304002', '72304002'),
        ('https://www.vimeo.com/72304002', '72304002'),
        ('http://player.vimeo.com/video/72304002', '72304002'),
        ('https://player.vimeo.com/video/72304002', '72304002'),
    )

    instance = VimeoBackend

    def test_vimeo_get_info_exception(self):
        with self.assertRaises(VideoDoesntExistException):
            backend = VimeoBackend('http://vimeo.com/123')
            backend.get_info()

    def test_get_thumbnail_url(self):
        backend = VimeoBackend('http://vimeo.com/72304002')
        self.assertEqual(backend.get_thumbnail_url(),
                         'http://b.vimeocdn.com/ts/446/150/446150690_640.jpg')


class SoundCloudBackendTestCase(BackendTestMixin, TestCase):
    urls = (
        ('https://soundcloud.com/glassnote/mumford-sons-i-will-wait', '67129237'),
        ('https://soundcloud.com/matej-roman/jaromir-nohavica-karel-plihal-mikymauz', '7834701'),
        ('https://soundcloud.com/beny97/sets/jaromir-nohavica-prazska', '960591'),
        ('https://soundcloud.com/corbel-keep/norah-jones-come-away-with', '22485933'),
    )

    instance = SoundCloudBackend

    def setUp(self):
        class FooBackend(SoundCloudBackend):
            url = 'foobar'

            def init(self, *args, **kwargs):
                return

            def get_info(self):
                return {
                    'width': 123,
                    'height': 321,
                    'thumbnail_url': 'xyz'
                }

        self.foo = FooBackend('abcd')

    def test_width(self):
        self.assertEqual(self.foo.width, 123)

    def test_height(self):
        self.assertEqual(self.foo.height, 321)

    def test_get_thumbnail_url(self):
        self.assertEqual(self.foo.get_thumbnail_url(), 'xyz')

    def test_get_embed_code(self):
        self.assertEqual(self.foo.get_embed_code(100, 200),
                         '<iframe width="100" height="321" src="foobar" '
                         'frameborder="0" allowfullscreen></iframe>')



from unittest import TestCase
from mock import Mock

from django.template import TemplateSyntaxError
from django.http import HttpRequest
from django.template.base import Template
from django.template.context import RequestContext

from embed_video.backends import YoutubeBackend, SoundCloudBackend
from embed_video.templatetags.embed_video_tags import VideoNode, \
        _embed_get_size


class EmbedVideoNodeTestCase(TestCase):
    def setUp(self):
        self.parser = Mock()
        self.token = Mock(methods=['split_contents'])

    def _grc(self):
        return RequestContext(HttpRequest())

    def test_embed(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {{ ytb|embed:'large' }}
            {% endvideo %}
        """)
        rendered = u'<iframe width="960" height="720" src="http://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" frameborder="0" allowfullscreen></iframe>'

        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_direct_embed(self):
        template = Template("""
            {% load embed_video_tags %}
            {{ 'http://www.youtube.com/watch?v=jsrRJyHBvzw'|embed:'large' }}
        """)
        rendered = u'<iframe width="960" height="720" src="http://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" frameborder="0" allowfullscreen></iframe>'

        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_embed_user_size(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {{ ytb|embed:'800x800' }}
            {% endvideo %}
        """)
        rendered = u'<iframe width="800" height="800" src="http://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" frameborder="0" allowfullscreen></iframe>'

        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_tag_youtube(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {{ ytb.url }}
            {% endvideo %}
        """)
        rendered = 'http://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque'

        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_tag_vimeo(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'https://vimeo.com/72304002' as vimeo %}
                {{ vimeo.url }}
            {% endvideo %}
        """)
        rendered = 'http://player.vimeo.com/video/72304002'

        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_tag_backend_variable_vimeo(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'https://vimeo.com/72304002' as vimeo %}
                {{ vimeo.backend }}
            {% endvideo %}
        """)
        rendered = 'VimeoBackend'
        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_tag_backend_variable_youtube(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvz' as youtube %}
                {{ youtube.backend }}
            {% endvideo %}
        """)
        rendered = 'YoutubeBackend'
        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_tag_backend_variable_soundcloud(self):
        template = Template("""
            {% load embed_video_tags %}
            {% video 'https://soundcloud.com/glassnote/mumford-sons-i-will-wait' as soundcloud %}
                {{ soundcloud.backend }}
            {% endvideo %}
        """)
        rendered = 'SoundCloudBackend'
        self.assertEqual(template.render(self._grc()).strip(), rendered)

    def test_syntax_error(self):
        self.token.split_contents.return_value = []

        try:
            VideoNode(self.parser, self.token)
        except TemplateSyntaxError:
            assert True

    def test_repr(self):
        self.token.split_contents.return_value = (
            'video', 'http://youtu.be/v/1234', 'as', 'myvideo'
        )
        self.parser.compile_filter.return_value = u'some_url'

        node = VideoNode(self.parser, self.token)
        self.assertEqual(str(node), '<VideoNode "some_url">')

    def test_videonode_iter(self):
        out = ['a', 'b', 'c', 'd']

        class FooNode(VideoNode):
            nodelist_file = out

            def __init__(self):
                pass

        node = FooNode()
        self.assertEqual(out, [x for x in node])





"""
Django app for easy embeding YouTube and Vimeo videos and music from
SoundCloud.
"""

VERSION = (0, 6, 'stable')


def get_release():
    return '.'.join(str(i) for i in VERSION[:3])


def get_version():
    """
    Returns only digit parts of version.
    """
    return '.'.join(str(i) for i in VERSION[:2])


__version__ = get_release()



import re
import requests
import json

try:
    import urlparse  # py2
except:
    import urllib.parse as urlparse  # py3

from django.utils.functional import cached_property

from .utils import import_by_path
from .settings import EMBED_VIDEO_BACKENDS, EMBED_VIDEO_CACHE

cache = None
if EMBED_VIDEO_CACHE:
    from django.core.cache import cache


class VideoDoesntExistException(Exception):
    pass


class UnknownBackendException(Exception):
    pass


class UnknownIdException(Exception):
    pass


def detect_backend(url):
    """
    Detect the right backend for given URL.
    """

    for backend_name in EMBED_VIDEO_BACKENDS:
        backend = import_by_path(backend_name)
        if backend.is_valid(url):
            return backend(url)

    raise UnknownBackendException


class VideoBackend(object):
    """
    Base backend, good to inherit.
    """

    re_code = None
    """
    Compiled regex (:py:func:`re.compile`) to search code in URL.

    Example: ``re.compile(r'myvideo\.com/\?code=(?P<code>\w+)')``
    """

    re_detect = None
    """
    Compilede regec (:py:func:`re.compile`) to detect, if input URL is valid
    for current backend.

    Example: ``re.compile(r'^http://myvideo\.com/.*')``
    """

    pattern_url = None
    """
    Pattern in which the code is inserted.

    Example: ``http://myvideo.com?code=%s``
    """

    pattern_thumbnail_url = None
    """
    Pattern in which the code is inserted to get thumbnail url.

    Example: ``http://static.myvideo.com/thumbs/%s``
    """

    def __init__(self, url):
        """
        First it tries to load data from cache and if it don't succeed, run
        :py:meth:`init` and then save it to cache.
        """
        if not self._restore_cache(url):
            self.init(url)
            self._set_cache(url)

    def _restore_cache(self, url):
        """
        Restore data from cache if cache is enabled.
        """
        if cache:
            data = cache.get(url)
            if data:
                self.__dict__ = data
                return True

    def _set_cache(self, url):
        """
        Set data to cache if cache is enabled.
        """
        if cache:
            cache.set(url, self.__dict__)

    def init(self, url):
        """
        Load all data (:py:meth:`get_code`, :py:meth:`get_url` etc.)
        """
        self._url = url
        self.backend = self.__class__.__name__
        self.code = self.get_code()
        self.url = self.get_url()

    @cached_property
    def thumbnail(self):
        return self.get_thumbnail_url()

    @cached_property
    def info(self):
        return self.get_info()

    @classmethod
    def is_valid(klass, url):
        """
        Class method to control if passed url is valid for current backend. By
        default it is done by :py:data:`re_detect` regex.
        """
        return True if klass.re_detect.match(url) else False

    def get_code(self):
        """
        Returns searched code from URL by :py:data:`re_code`.
        """
        match = self.re_code.search(self._url)
        if match:
            return match.group('code')

    def get_url(self):
        """
        Returns URL folded from :py:data:`pattern_url` and parsed code.
        """
        return self.pattern_url % self.code

    def get_thumbnail_url(self):
        """
        Returns thumbnail URL folded from :py:data:`pattern_thumbnail_url` and
        parsed code.
        """
        return self.pattern_thumbnail_url % self.code

    def get_embed_code(self, width, height):
        """
        Returns embed code.
        """
        return '<iframe width="%(width)d" height="%(height)d" ' \
                'src="%(url)s" frameborder="0" allowfullscreen>' \
                '</iframe>' % {
                    'url': self.url,
                    'width': width,
                    'height': height,
                }


class YoutubeBackend(VideoBackend):
    """
    Backend for YouTube URLs.
    """
    re_detect = re.compile(
        r'^(http(s)?://)?(www\.)?youtu(\.?)be(\.com)?/.*', re.I
    )

    re_code = re.compile(
        r'''youtu(\.?)be(\.com)?/  # match youtube's domains
            (embed/)?  # match the embed url syntax
            (v/)?
            (watch\?v=)?  # match the youtube page url
            (ytscreeningroom\?v=)?
            (feeds/api/videos/)?
            (user\S*[^\w\-\s])?
            (\S*[^\w\-\s])?
            (?P<code>[\w\-]{11})[a-z0-9;:@?&%=+/\$_.-]*  # match and extract
        ''',
        re.I | re.X
    )

    pattern_url = 'http://www.youtube.com/embed/%s?wmode=opaque'
    pattern_thumbnail_url = 'http://img.youtube.com/vi/%s/hqdefault.jpg'

    def get_code(self):
        code = super(YoutubeBackend, self).get_code()

        if not code:
            parse_data = urlparse.urlparse(self._url)

            try:
                code = urlparse.parse_qs(parse_data.query)['v'][0]
            except KeyError:
                raise UnknownIdException

        return code


class VimeoBackend(VideoBackend):
    """
    Backend for Vimeo URLs.
    """
    re_detect = re.compile(
        r'^(http(s)?://)?(www\.)?(player\.)?vimeo\.com/.*', re.I
    )
    re_code = re.compile(r'''vimeo\.com/(video/)?(?P<code>[0-9]+)''', re.I)
    pattern_url = 'http://player.vimeo.com/video/%s'
    pattern_info = 'http://vimeo.com/api/v2/video/%s.json'

    def init(self, url):
        self._url = url
        self.code = self.get_code()

        super(VimeoBackend, self).init(url)

    def get_info(self):
        try:
            response = requests.get(self.pattern_info % self.code)
            return json.loads(response.text)[0]
        except ValueError:
            raise VideoDoesntExistException()

    def get_thumbnail_url(self):
        return self.info.get('thumbnail_large')


class SoundCloudBackend(VideoBackend):
    """
    Backend for SoundCloud URLs.
    """
    base_url = 'http://soundcloud.com/oembed'

    re_detect = re.compile(r'^(http(s)?://(www\.)?)?soundcloud\.com/.*', re.I)
    re_code = re.compile(r'src=".*%2F(?P<code>\d+)&show_artwork.*"', re.I)
    re_url = re.compile(r'src="(?P<url>.*?)"', re.I)

    @cached_property
    def width(self):
        return self.info.get('width')

    @cached_property
    def height(self):
        return self.info.get('height')

    def get_info(self):
        params = {
            'format': 'json',
            'url': self._url,
        }
        r = requests.get(self.base_url, data=params)
        return json.loads(r.text)

    def get_thumbnail_url(self):
        return self.info.get('thumbnail_url')

    def get_url(self):
        match = self.re_url.search(self.info.get('html'))
        return match.group('url')

    def get_code(self):
        match = self.re_code.search(self.info.get('html'))
        return match.group('code')

    def get_embed_code(self, width, height):
        return super(SoundCloudBackend, self). \
                get_embed_code(width=width, height=self.height)



import sys
from unittest import TestCase

from django.core.exceptions import ImproperlyConfigured

from embed_video.utils import import_by_path


class ModuleImportTestCase(TestCase):
    """
    Taken from Django:

    https://github.com/django/django/blob/master/tests/utils_tests/test_module_loading.py
    """

    def test_incorrect_path(self):
        self.assertRaises(ImproperlyConfigured, import_by_path, 'wrongpath')

    def test_incorrect_classname(self):
        self.assertRaises(ImproperlyConfigured, import_by_path,
                          'embed_video.foo')

    def test_import_by_path(self):
        cls = import_by_path(
            'embed_video.utils.import_by_path')
        self.assertEqual(cls, import_by_path)

        # Test exceptions raised
        for path in ('no_dots_in_path', 'unexistent.path',
                'utils_tests.unexistent'):
            self.assertRaises(ImproperlyConfigured, import_by_path, path)

        with self.assertRaises(ImproperlyConfigured) as cm:
            import_by_path('unexistent.module.path', error_prefix="Foo")
        self.assertTrue(str(cm.exception).startswith('Foo'))



import re

from embed_video.backends import VideoBackend


class CustomBackend(VideoBackend):
    re_detect = re.compile(r'http://myvideo\.com/[0-9]+')
    re_code = re.compile(r'http://myvideo\.com/(?P<code>[0-9]+)')

    pattern_url = 'http://play.myvideo.com/c/%s/'
    pattern_thumbnail_url = 'http://thumb.myvideo.com/c/%s/'



from unittest import TestCase

from django.forms import TextInput

from embed_video.admin import AdminVideoWidget, AdminVideoMixin
from embed_video.backends import VimeoBackend
from embed_video.fields import EmbedVideoField, EmbedVideoFormField


class AdminVideoWidgetTestCase(TestCase):
    def test_size(self):
        widget = AdminVideoWidget()
        self.assertTrue('size' in widget.attrs)

    def test_overwrite_size(self):
        widget = AdminVideoWidget(attrs={'size': 0})
        self.assertEqual(widget.attrs['size'], 0)

    def test_add_to_attrs(self):
        widget = AdminVideoWidget(attrs={'name': 'foo'})
        self.assertEqual(widget.attrs['name'], 'foo')
        self.assertTrue('size' in widget.attrs)

    def test_render_empty_value(self):
        widget = AdminVideoWidget(attrs={'size': '0'})
        self.assertEqual(widget.render('foo'),
                         '<input name="foo" size="0" type="text" />')

    def test_render(self):
        backend = VimeoBackend('https://vimeo.com/1')
        widget = AdminVideoWidget(attrs={'size': '0'})
        widget.output_format = '{video}{input}'

        self.assertEqual(
            widget.render('foo', backend.url, size=(100, 100)),
            backend.get_embed_code(100, 100)
            + '<input name="foo" size="0" type="text" value="%s" />'
                % backend.url
        )

    def test_incorrect_value(self):
        widget = AdminVideoWidget()
        self.assertEqual(widget.render('foo', 'abcd'),
             '<input name="foo" size="40" type="text" value="abcd" />')


class AdminVideoMixinTestCase(TestCase):
    def test_embedvideofield(self):
        foo = EmbedVideoField()
        mixin = AdminVideoMixin()
        self.assertTrue(isinstance(mixin.formfield_for_dbfield(foo),
                                   EmbedVideoFormField))

    def test_other_fields(self):
        class Parent(object):
            def formfield_for_dbfield(*args, **kwargs):
                raise Exception

        class MyAdmin(AdminVideoMixin, Parent):
            pass

        myadmin = MyAdmin()
        self.assertRaises(Exception, myadmin.formfield_for_dbfield, 'foo')



from django.template import Library, Node, TemplateSyntaxError
from django.utils.safestring import mark_safe

from ..backends import detect_backend, VideoBackend

register = Library()


@register.tag('video')
class VideoNode(Node):
    """
    Template tag ``video``. It gives access to all
    :py:class:`~embed_video.backends.VideoBackend` variables.

    Usage:

    .. code-block:: html+django

        {% video URL as VAR %}
            ...
        {% endvideo %}

    Example:

    .. code-block:: html+django

        {% video item.video as my_video %}
            URL: {{ my_video.url }}
            Thumbnail: {{ my_video.thumbnail }}
            Backend: {{ my_video.backend }}
        {% endvideo %}

    """
    error_msg = ('Syntax error. Expected: ``video source as var``')

    def __init__(self, parser, token):
        bits = token.split_contents()

        if len(bits) < 4 or bits[-2] != 'as':
            raise TemplateSyntaxError(self.error_msg)

        self.url = parser.compile_filter(bits[1])
        self.as_var = bits[-1]

        self.nodelist_file = parser.parse(('endvideo',))
        parser.delete_first_token()

    def render(self, context):
        url = self.url.resolve(context)
        context.push()
        context[self.as_var] = detect_backend(url)
        output = self.nodelist_file.render(context)
        context.pop()
        return output

    def __iter__(self):
        for node in self.nodelist_file:
            yield node

    def __repr__(self):
        return '<VideoNode "%s">' % (self.url)


@register.filter(is_safe=True)
def embed(backend, size='small'):
    """
    Shortcut for :py:func:`VideoNode` tag.

    Usage:

    .. code-block:: html+django

        {{ URL|embed:SIZE }}

    Example:

    .. code-block:: html+django

        {{ 'http://www.youtube.com/watch?v=guXyvo2FfLs'|embed:'large' }}

    Predefined sizes:

        ======== ======== =========
        size     width    height
        ======== ======== =========
        tiny     420      315
        small    480      360
        medium   640      480
        large    960      720
        huge     1280     960
        ======== ======== =========

    You can also use custom size - in format ``WIDTHxHEIGHT``
    (eg. ``500x400``).

    """

    if not isinstance(backend, VideoBackend):
        backend = detect_backend(backend)

    _size = _embed_get_size(size)

    return mark_safe(backend.get_embed_code(width=_size[0],
                                            height=_size[1]))


def _embed_get_size(size):
    sizes = {
        'tiny': (420, 315),
        'small': (480, 360),
        'medium': (640, 480),
        'large': (960, 720),
        'huge': (1280, 960),
    }

    if size in sizes:
        return sizes[size]
    elif 'x' in size:
        return [int(x) for x in size.split('x')]




from django.conf import settings


EMBED_VIDEO_BACKENDS = getattr(settings, 'EMBED_VIDEO_BACKENDS', (
    'embed_video.backends.YoutubeBackend',
    'embed_video.backends.VimeoBackend',
    'embed_video.backends.SoundCloudBackend',
))


EMBED_VIDEO_CACHE = getattr(settings, 'EMBED_VIDEO_CACHE', True)

EMBED_VIDEO_CACHE_TIMEOUT = getattr(settings, 'EMBED_VIDEO_CACHE_TIMEOUT',
                                    3600)



from unittest import TestCase

from embed_video.backends import detect_backend

from .custom_backend import CustomBackend


class CustomBackendTestCase(TestCase):
    def setUp(self):
        self.backend = detect_backend('http://myvideo.com/1530')

    def test_detect_backend(self):
        self.assertIsInstance(self.backend, CustomBackend)

    def test_code(self):
        self.assertEqual(self.backend.code, '1530')

    def test_url(self):
        self.assertEqual(self.backend.get_url(),
                         'http://play.myvideo.com/c/1530/')

    def test_thumbnail(self):
        self.assertEqual(self.backend.get_thumbnail_url(),
                         'http://thumb.myvideo.com/c/1530/')




from mock import patch
from unittest import TestCase

from django.forms import ValidationError
from ..fields import EmbedVideoField, EmbedVideoFormField
from ..backends import UnknownBackendException, UnknownIdException


class EmbedVideoFieldTestCase(TestCase):
    def setUp(self):
        self.field = EmbedVideoField()

    def test_formfield_form_class(self):
        self.assertIsInstance(self.field.formfield(),
                              EmbedVideoFormField)

    def test_south(self):
        self.assertEqual(self.field.south_field_triple(),
                         (
                             'embed_video.fields.EmbedVideoField',
                             [], {'max_length': '200'}
                         ))


class EmbedVideoFormFieldTestCase(TestCase):
    def setUp(self):
        self.formfield = EmbedVideoFormField()

    def test_validation_unknownbackend(self):
        with patch('embed_video.fields.detect_backend') as mock_detect_backend:
            mock_detect_backend.return_value = True
            mock_detect_backend.side_effect = UnknownBackendException
            self.assertRaises(ValidationError, self.formfield.validate,
                              ('http://youtube.com/v/123/',))

    def test_validation_unknownid(self):
        with patch('embed_video.fields.detect_backend') as mock_detect_backend:
            mock_detect_backend.return_value = True
            mock_detect_backend.side_effect = UnknownIdException
            self.assertRaises(ValidationError, self.formfield.validate,
                              ('http://youtube.com/v/123/',))

    def test_validation_correct(self):
        url = 'http://my-testing.url.com'
        with patch('embed_video.fields.detect_backend') as mock_detect_backend:
            mock_detect_backend.return_value = True
            self.assertEqual(url, self.formfield.validate(url))



import os

from django.conf.global_settings import *

DEBUG = True
SECRET_KEY = 'testing_key123'

STATIC_ROOT = MEDIA_ROOT = os.path.join(os.path.dirname(__file__), 'static')
STATIC_URL = MEDIA_URL = '/static/'

INSTALLED_APPS = (
    'embed_video',
)


EMBED_VIDEO_BACKENDS = (
    'embed_video.backends.YoutubeBackend',
    'embed_video.backends.VimeoBackend',
    'embed_video.backends.SoundCloudBackend',
    'embed_video.tests.custom_backend.CustomBackend',
)


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'less': {
            'handlers': ['console'],
            'level': 'DEBUG',
        },
    }
}
